% Here we compute the gradient with the adjoint sensitivity analysis.
% In particular, we discretize and differentiate the discretized form.

function [grad]=DiscrDiff(m,cs,ksl,cd,kd,alpha,Ndamper,Ndof,Nh,gammad,dis_all,r,q)
global coeff_alpha ratio Ns Fy IdNh T

%% Evaluate the structural response (max drift)

load analysis

time=time1;
sinBeta=sind(gammad);
IdNd=eye(Ndamper);
IdNdof=eye(Ndof);
IdNh=eye(Nh);

dis=dis1;
d=H*dis;
d_bar=((dis_all.^-1)*eye(2))*d;
d_barMAX=max(abs(d_bar),[],2);
d_cap=diag(d_barMAX.^-1)*d_bar;
int=d_cap.^r;
tf=max(time,[],2);
N=size(time,2);
w=ones(1,N);
w(1)=w(1)*(time(2)-time(1))/2;
w(N)=w(end)*(time(N)-time(N-1))/2;
for i=2:N-1
    dt1=time(i)-time(i-1);
    dt2=time(i+1)-time(i);
    w(i)=w(i)*(dt1+dt2)/2;
end

dcr=w*int';
dcr=dcr';
dc_vec=diag(d_barMAX)*((dcr./tf).^(1/r));%Vector!!
dcvecMAX=max(abs(dc_vec),[],1);
one=[1;1];
dc_vec=dc_vec./dcvecMAX;
num=one'*(diag(dc_vec)^(q+1))*one;
den=one'*(diag(dc_vec)^(q))*one;
dc=dcvecMAX*(num)/(den);



%% Gradient of the drift constraint
%  Application to the discretized problem of an adjoint method

kd1=zeros(Ndamper,N-1);
kd2=zeros(Ndamper,N-1);
kd3=zeros(Ndamper,N-1);
kd4=zeros(Ndamper,N-1);
kkd1=zeros(Ndamper,N-1);
kkd2=zeros(Ndamper,N-1);
kkd3=zeros(Ndamper,N-1);
kkd4=zeros(Ndamper,N-1);

ks1=zeros(Nh,N-1);
ks2=zeros(Nh,N-1);
ks3=zeros(Nh,N-1);
ks4=zeros(Nh,N-1);

cdinv=cd\eye(2);

for i=1:1:N-1
    
    dt=time(i+1)-time(i);
    vel_half=(vel(:,i+1)+vel(:,i))./2;
    
    ks1(:,i) = ksl*(eye(Nh)-0.5*diag(abs(diag((2*Fy).^-1)*Fcol(:,i)).^Ns)*diag(sign(diag(Fcol(:,i))*T*vel(:,i))+1) )*T*vel(:,i);
    ks2(:,i) = ksl*(eye(Nh)-0.5*diag(abs(diag((2*Fy).^-1)*(Fcol(:,i)+dt*ks1(:,i)/2)).^Ns)*diag(sign(diag(Fcol(:,i)+dt*ks1(:,i)/2)*T*vel_half)+1) )*T*vel_half;
    ks3(:,i) = ksl*(eye(Nh)-0.5*diag(abs(diag((2*Fy).^-1)*(Fcol(:,i)+dt*ks2(:,i)/2)).^Ns)*diag(sign(diag(Fcol(:,i)+dt*ks2(:,i)/2)*T*vel_half)+1) )*T*vel_half;
    ks4(:,i) = ksl*(eye(Nh)-0.5*diag(abs(diag((2*Fy).^-1)*(Fcol(:,i)+dt*ks3(:,i))).^Ns)*diag(sign(diag(Fcol(:,i)+dt*ks3(:,i))*T*vel(:,i+1))+1) )*T*vel(:,i+1);
    
    bordello1=( T*vel(:,i)*sinBeta - ((cdinv*diag(abs(Fd(:,i)))).^(1/alpha))*sign(Fd(:,i)));
    kd1(:,i) = kd*bordello1;
    kkd1(:,i) = bordello1;
    
    bordello2=( T*vel_half*sinBeta - ((cdinv*diag(abs(Fd(:,i)+dt*kd1(:,i)/2))).^(1/alpha))*sign(Fd(:,i)+dt*kd1(:,i)/2));
    kd2(:,i) = kd*bordello2;
    kkd2(:,i) = bordello2;
    
    bordello3=( T*vel_half*sinBeta - ((cdinv*diag(abs(Fd(:,i)+dt*kd2(:,i)/2))).^(1/alpha))*sign(Fd(:,i)+dt*kd2(:,i)/2));
    kd3(:,i) = kd*bordello3;
    kkd3(:,i) = bordello3;
    
    bordello4=( T*vel(:,i+1)*sinBeta - ((cdinv*diag(abs(Fd(:,i)+dt*kd3(:,i)))).^(1/alpha))*sign(Fd(:,i)+dt*kd3(:,i)));
    kd4(:,i) = kd*bordello4;
    kkd4(:,i) = bordello4;
    
end

dt=time(N)-time(N-1);


AGrn=zeros(5*Ndamper+5*Nh+3*Ndof,5*Ndamper+5*Nh+3*Ndof);
bn=zeros(5*Ndamper+5*Nh+3*Ndof,1);

AGrn(1:Ndamper,1:Ndamper)=-eye(Ndamper);
AGrn(1:Ndamper,4*Ndamper+1:5*Ndamper)=dt/6*eye(Ndamper);

AGrn(Ndamper+1:2*Ndamper,1:Ndamper)=-funcG(Fd(:,N-1)+dt*kd3(:,N-1))'*kd*dt;
AGrn(Ndamper+1:2*Ndamper,Ndamper+1:2*Ndamper)=-eye(Ndamper);
AGrn(Ndamper+1:2*Ndamper,4*Ndamper+1:5*Ndamper)=dt/3*eye(Ndamper);

AGrn(2*Ndamper+1:3*Ndamper,Ndamper+1:2*Ndamper)=-funcG(Fd(:,N-1)+dt*kd2(:,N-1)/2)'*kd*(dt/2);
AGrn(2*Ndamper+1:3*Ndamper,2*Ndamper+1:3*Ndamper)=-eye(Ndamper);
AGrn(2*Ndamper+1:3*Ndamper,4*Ndamper+1:5*Ndamper)=dt/3*eye(Ndamper);

AGrn(3*Ndamper+1:4*Ndamper,2*Ndamper+1:3*Ndamper)=-funcG(Fd(:,N-1)+dt*kd1(:,N-1)/2)'*kd*(dt/2);
AGrn(3*Ndamper+1:4*Ndamper,3*Ndamper+1:4*Ndamper)=-eye(Ndamper);
AGrn(3*Ndamper+1:4*Ndamper,4*Ndamper+1:5*Ndamper)=dt/6*eye(Ndamper);

AGrn(4*Ndamper+1:5*Ndamper,4*Ndamper+1:5*Ndamper)=-eye(Ndamper);
AGrn(4*Ndamper+1:5*Ndamper,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=T*sinBeta;

AGrn(5*Ndamper+1:5*Ndamper+Nh,5*Ndamper+1:5*Ndamper+Nh)=-eye(Nh);
AGrn(5*Ndamper+1:5*Ndamper+Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/6*eye(Nh);

AGrn(5*Ndamper+Nh+1:5*Ndamper+2*Nh,5*Ndamper+Nh+1:5*Ndamper+2*Nh)=-eye(Nh);
AGrn(5*Ndamper+Nh+1:5*Ndamper+2*Nh,5*Ndamper+1:5*Ndamper+Nh)=-funcM(Fcol(:,N-1)+dt*ks3(:,N-1),T*vel(:,N))*dt;
AGrn(5*Ndamper+Nh+1:5*Ndamper+2*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/3*eye(Nh);

AGrn(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,5*Ndamper+2*Nh+1:5*Ndamper+3*Nh)=-eye(Nh);
AGrn(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,5*Ndamper+Nh+1:5*Ndamper+2*Nh)=-funcM(Fcol(:,N-1)+dt*ks2(:,N-1)/2,0.5*T*(vel(:,N)+vel(:,N-1)))*dt/2;
AGrn(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/3*eye(Nh);

AGrn(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,5*Ndamper+3*Nh+1:5*Ndamper+4*Nh)=-eye(Nh);
AGrn(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,5*Ndamper+2*Nh+1:5*Ndamper+3*Nh)=-funcM(Fcol(:,N-1)+dt*ks1(:,N-1)/2,0.5*T*(vel(:,N)+vel(:,N-1)))*dt/2;
AGrn(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/6*eye(Nh);

AGrn(5*Ndamper+4*Nh+1:5*Ndamper+5*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=-eye(Nh);
AGrn(5*Ndamper+4*Nh+1:5*Ndamper+5*Nh,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=T;

AGrn(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof)=(dt/2)*eye(Ndof);
AGrn(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof)=((dt^2)/4)*eye(Ndof);
AGrn(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=m';

AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,1:Ndamper)=T'*kd*sinBeta;
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,Ndamper+1:2*Ndamper)=T'*(kd./2)*sinBeta;
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,2*Ndamper+1:3*Ndamper)=T'*(kd./2)*sinBeta;
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+1:5*Ndamper+Nh)= funcO(Fcol(:,N-1)+dt*ks3(:,N-1),T*vel(:,N));
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+Nh+1:5*Ndamper+2*Nh)= 0.5*funcO(Fcol(:,N-1)+dt*ks2(:,N-1)/2,0.5*T*(vel(:,N)+vel(:,N-1)));
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+2*Nh+1:5*Ndamper+3*Nh)= 0.5*funcO(Fcol(:,N-1)+dt*ks1(:,N-1)/2,0.5*T*(vel(:,N)+vel(:,N-1)));
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof)=-eye(Ndof);
AGrn(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=cs';



AGrn(5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof,5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof)=-eye(Ndof);


bn(5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=-H'*diag(d_barMAX)*diag(((sum((d_cap.^r).*[w;w],2))/tf).^((1-r)/r))*...
    (1/tf)*w(end)*(diag(d_cap(:,N)).^(r-1))*diag(d_barMAX.^-1)*((dis_all.^-1)*eye(2))*...
    (1/den^2)*(den*(q+1)*(diag(dc_vec).^q)*one - num*(q)*(diag(dc_vec).^(q-1))*one);

x=zeros(5*Ndamper+5*Nh+3*Ndof,N); % N is the length of the time vector
x(:,N)=AGrn\bn;


AGr=zeros(5*Ndamper+5*Nh+3*Ndof,5*Ndamper+5*Nh+3*Ndof);
b=zeros(5*Ndamper+5*Nh+3*Ndof,1);



%--------------------------------------------------------------------------
% Calculate here the parts of the matrices that do not change in the iterations



AGr(1:Ndamper,1:Ndamper)=-eye(Ndamper);
AGr(1:Ndamper,4*Ndamper+1:5*Ndamper)=dt/6*eye(Ndamper);
AGr(Ndamper+1:2*Ndamper,Ndamper+1:2*Ndamper)=-eye(Ndamper);
AGr(2*Ndamper+1:3*Ndamper,2*Ndamper+1:3*Ndamper)=-eye(Ndamper);
AGr(3*Ndamper+1:4*Ndamper,3*Ndamper+1:4*Ndamper)=-eye(Ndamper);
AGr(4*Ndamper+1:5*Ndamper,4*Ndamper+1:5*Ndamper)=-eye(Ndamper);
AGr(5*Ndamper+1:5*Ndamper+Nh,5*Ndamper+1:5*Ndamper+Nh)=-eye(Nh);
AGr(5*Ndamper+Nh+1:5*Ndamper+2*Nh,5*Ndamper+Nh+1:5*Ndamper+2*Nh)=-eye(Nh);
AGr(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,5*Ndamper+2*Nh+1:5*Ndamper+3*Nh)=-eye(Nh);
AGr(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,5*Ndamper+3*Nh+1:5*Ndamper+4*Nh)=-eye(Nh);
AGr(5*Ndamper+4*Nh+1:5*Ndamper+5*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=-eye(Nh);
AGr(5*Ndamper+4*Nh+1:5*Ndamper+5*Nh,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=T;
AGr(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=m';
AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,1:Ndamper)=T'*kd*sinBeta;
AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,Ndamper+1:2*Ndamper)=T'*(kd./2)*sinBeta;
AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,2*Ndamper+1:3*Ndamper)=T'*(kd./2)*sinBeta;
AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof)=-eye(Ndof);
AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=cs';
AGr(5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof,5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof)=-eye(Ndof);

% These refer to te b vector:
AA=H'*diag(d_barMAX)*diag(((sum((d_cap.^r).*[w;w],2))/tf).^((1-r)/r));
BB=(1/den^2)*(den*(q+1)*(diag(dc_vec).^q)*one - num*(q)*(diag(dc_vec).^(q-1))*one);
Id2=eye(Ndof);
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
for i=N-1:-1:2
    dt=time(i)-time(i-1);
    
    AGr(Ndamper+1:2*Ndamper,1:Ndamper)=-funcG(Fd(:,i-1)+dt*kd3(:,i-1))'*kd*dt;
    AGr(Ndamper+1:2*Ndamper,4*Ndamper+1:5*Ndamper)=dt/3*IdNd;
    
    AGr(2*Ndamper+1:3*Ndamper,Ndamper+1:2*Ndamper)=-funcG(Fd(:,i-1)+dt*kd2(:,i-1)/2)'*kd*(dt/2);
    AGr(2*Ndamper+1:3*Ndamper,4*Ndamper+1:5*Ndamper)=dt/3*IdNd;
    
    AGr(3*Ndamper+1:4*Ndamper,2*Ndamper+1:3*Ndamper)=-funcG(Fd(:,i-1)+dt*kd1(:,i-1)/2)'*kd*(dt/2);
    AGr(3*Ndamper+1:4*Ndamper,4*Ndamper+1:5*Ndamper)=dt/6*IdNd;
    
    AGr(4*Ndamper+1:5*Ndamper,5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=T*sinBeta;
    
    AGr(5*Ndamper+1:5*Ndamper+Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/6*IdNh;
    
    AGr(5*Ndamper+Nh+1:5*Ndamper+2*Nh,5*Ndamper+1:5*Ndamper+Nh)=-funcM(Fcol(:,i-1)+dt*ks3(:,i-1),T*vel(:,i))*dt;
    AGr(5*Ndamper+Nh+1:5*Ndamper+2*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/3*IdNh;
    
    AGr(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,5*Ndamper+Nh+1:5*Ndamper+2*Nh)=-funcM(Fcol(:,i-1)+dt*ks2(:,i-1)/2,0.5*T*(vel(:,i)+vel(:,i-1)))*dt/2;
    AGr(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/3*IdNh;
    
    AGr(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,5*Ndamper+2*Nh+1:5*Ndamper+3*Nh)=-funcM(Fcol(:,i-1)+dt*ks1(:,i-1)/2,0.5*T*(vel(:,i)+vel(:,i-1)))*dt/2;
    AGr(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=dt/6*IdNh;
    
    AGr(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof)=(dt/2)*IdNdof;
    AGr(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof)=((dt^2)/4)*IdNdof;
    
    AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+1:5*Ndamper+Nh)= funcO(Fcol(:,i-1)+dt*ks3(:,i-1),T*vel(:,i));
    AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+Nh+1:5*Ndamper+2*Nh)= 0.5*funcO(Fcol(:,i-1)+dt*ks2(:,i-1)/2,0.5*T*(vel(:,i)+vel(:,i-1)));
    AGr(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,5*Ndamper+2*Nh+1:5*Ndamper+3*Nh)= 0.5*funcO(Fcol(:,i-1)+dt*ks1(:,i-1)/2,0.5*T*(vel(:,i)+vel(:,i-1)));
    
    dtt=time(i+1)-time(i);
    b(4*Ndamper+1:5*Ndamper)=-x(4*Ndamper+1:5*Ndamper,i+1) + funcG(Fd(:,i))'*kd*x(3*Ndamper+1:4*Ndamper,i+1) + funcG(Fd(:,i)+dtt*kd1(:,i)/2)'*kd*x(2*Ndamper+1:3*Ndamper,i+1)+...
        funcG(Fd(:,i)+dtt*kd2(:,i)/2)'*kd*x(Ndamper+1:2*Ndamper,i+1) + funcG(Fd(:,i)+dtt*kd3(:,i))'*kd*x(1:Ndamper,i+1);
    
    b(5*Ndamper+4*Nh+1:5*Ndamper+5*Nh)=-x(5*Ndamper+4*Nh+1:5*Ndamper+5*Nh,i+1)+funcM(Fcol(:,i),T*vel(:,i))*x(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,i+1)...
        +funcM(Fcol(:,i)+dtt*ks1(:,i)/2,0.5*T*(vel(:,i+1)+vel(:,i)))*x(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,i+1)...
        +funcM(Fcol(:,i)+dtt*ks2(:,i)/2,0.5*T*(vel(:,i+1)+vel(:,i)))*x(5*Ndamper+Nh+1:5*Ndamper+2*Nh,i+1)...
        +funcM(Fcol(:,i)+dtt*ks3(:,i),T*vel(:,i+1))*x(5*Ndamper+1:5*Ndamper+Nh,i+1);
    
    b(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof)=-x(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,i+1)*(dtt/2)-x(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,i+1)*(dtt^2)/4;
    
    b(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof)=-x(5*Ndamper+5*Nh+1:5*Ndamper+5*Nh+Ndof,i+1) -x(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,i+1)*dtt...
        -funcO(Fcol(:,i),T*vel(:,i))*x(5*Ndamper+3*Nh+1:5*Ndamper+4*Nh,i+1)-0.5*funcO(Fcol(:,i)+dtt*ks1(:,i)/2,0.5*T*(vel(:,i+1)+vel(:,i)))*x(5*Ndamper+2*Nh+1:5*Ndamper+3*Nh,i+1)...
        - 0.5*funcO(Fcol(:,i)+dtt*ks2(:,i)/2,0.5*T*(vel(:,i+1)+vel(:,i)))*x(5*Ndamper+Nh+1:5*Ndamper+2*Nh,i+1)...
        - T'*kd*x(3*Ndamper+1:4*Ndamper,i+1)*sinBeta...
        -T'*(kd/2)*x(2*Ndamper+1:3*Ndamper,i+1)*sinBeta -T'*(kd/2)*x(Ndamper+1:2*Ndamper,i+1)*sinBeta;
    
    b(5*Ndamper+5*Nh+2*Ndof+1:5*Ndamper+5*Nh+3*Ndof)=-x(5*Ndamper+5*Nh+Ndof+1:5*Ndamper+5*Nh+2*Ndof,i+1)...
        -AA*(1/tf)*w(i)*(diag(d_cap(:,i)).^(r-1))*diag(d_barMAX.^-1)*((dis_all.^-1)*Id2)*BB;
    
    x(:,i)=AGr\b;
    
end

grad=zeros(1,Ndof);

Drho=ratio*eye(2);
for i=2:N
    dt=time(i)-time(i-1);
    grad=grad + x(3*Ndamper+1:4*Ndamper,i)'*(Drho*diag(kkd1(:,i-1)) +((1/coeff_alpha)*(kd)*diag(diag(cd).^(-(1/coeff_alpha +1)))*diag(abs(Fd(:,i-1)).^(1/alpha))*diag(sign(Fd(:,i-1)))))+...
        x(2*Ndamper+1:3*Ndamper,i)'*(Drho*diag(kkd2(:,i-1)) +(1/coeff_alpha)*(kd)*diag(diag(cd).^(-(1/coeff_alpha +1)))*diag(abs(Fd(:,i-1)+dt*kd1(:,i-1)/2).^(1/alpha))*diag(sign(Fd(:,i-1)+dt*kd1(:,i-1)/2)))+...
        x(Ndamper+1:2*Ndamper,i)'*(Drho*diag(kkd3(:,i-1)) +(1/coeff_alpha)*(kd)*diag(diag(cd).^(-(1/coeff_alpha +1)))*diag(abs(Fd(:,i-1)+dt*kd2(:,i-1)/2).^(1/alpha))*diag(sign(Fd(:,i-1)+dt*kd2(:,i-1)/2)))+...
        x(1:Ndamper,i)'*(Drho*diag(kkd4(:,i-1)) +(1/coeff_alpha)*(kd)*diag((diag(cd)).^(-(1/coeff_alpha +1)))*diag(abs(Fd(:,i-1)+dt*kd3(:,i-1)).^(1/alpha))*diag(sign(Fd(:,i-1)+dt*kd3(:,i-1))));
end

end

%% Functions used in the matrix A

function [y]=funcG(x)
global cd coeff_alpha
CD=diag(cd);
y=(1/coeff_alpha)*diag((CD).^(-1/coeff_alpha))*diag(x)*diag(abs(x).^(1/coeff_alpha-2))*diag(sign(x));
end

function [z]=funcM(x,y)
global Ns ksl Fy
z=0.5*Ns*ksl*diag((2*Fy).^-1)*diag(sign(diag((2*Fy).^-1)*x))*diag(abs(diag((2*Fy).^-1)*x).^(Ns-1))*diag(sign(diag(x)*y)+1)*diag(y);
end

function [z]=funcO(x,y)
global Ns ksl Fy T IdNh
z=T'*ksl*( IdNh-0.5*diag(abs(diag((2*Fy).^-1)*x).^Ns)*diag(sign(diag(x)*y)+diag(IdNh)) );
end







