function [q,qq,time,P,Fd,Fcol]=timestep_nonlin(m,cs,cd,ks,ksl,kd,acc,time,q0,qq0,Fd0,alpha,gamma)

global Ns Fy
load e
load T

ksl_Siv=ksl;

P=-m*e*acc;
qqq0=m\(P(:,1)-cs*qq0-T'*Fd0*sind(gamma)-ks*q0); %acc iniziale

Dt=time(2)-time(1);
q=zeros(size(q0,1),size(time,2));
qq=zeros(size(q0,1),size(time,2));
qqq=zeros(size(q0,1),size(time,2));
Fd=zeros(size(Fd0,1),size(time,2));
Fcol=zeros(size(T,1),size(time,2));
residuo = zeros(size(Fd0,1),size(time,2));

% Initialize the state variables
qi=q0;
qqi=qq0;
qqqi=qqq0;
Fcoli=ksl*T*q0;

q(:,1)=q0;
qq(:,1)=qq0;
qqq(:,1)=qqq0;
Fcol(:,1)=Fcoli;
Fd(:,1)=Fd0;
Fdi=Fd0;
Fdj=Fdi;
Fcolj=Fcoli;

% Initialize the time steps counter
i=1;
% Tollerance for the residual of the total equilibrium
Tol=1e-6;
% Exponent for the stiffness by Sivaselvan
expN=Ns;


hwait=waitbar(0,'Integrating the equations of motion...');

while 1<2 % Counting with i
    
    Dt=time(i+1)-time(i);
    
    Kcap=(4*m/(Dt^2) + 2*cs/Dt +ks);
    invKcap=Kcap\eye(2);
    
    qqqj=qqqi;
    qj=qi+Dt*qqi+((Dt^2)/4)*qqqi+((Dt^2)/4)*qqqj;
    qqj=qqi+(Dt/2)*qqqi+(Dt/2)*qqqj;
    dq=qj-qi;
    
    j=0;
    
    vDrift_i=T*qqi;
    while 1<2 % Counting with j
        
        
        vDrift_j=T*qqj;
        for jj=1:size(Fcolj,1)
            [fcj]=RK_s(Fcoli(jj)/2,vDrift_i(jj),vDrift_j(jj),ksl(jj,jj)/2,Fy(jj),expN,Dt);
            Fcolj(jj)=fcj*2;
        end
        
        for jj=1:size(Fdj,1)
            [fdj]=RK_d(Fdi(jj),vDrift_i(jj)*sind(gamma),vDrift_j(jj)*sind(gamma),alpha(jj),kd(jj,jj),cd(jj,jj),Dt);
            Fdj(jj)=fdj;
        end

        
        R=P(:,i+1)-m*qqqj-cs*qqj-T'*Fcolj-T'*Fdj*sind(gamma);
        
        
        if max(abs(R))<=Tol
            break
        else
            dq = invKcap*R;
            qj=qj+dq;
            qqj=qqj+2*dq/Dt;
            qqqj=qqqj+4*dq/(Dt^2);
        end
        
        j=j+1;
        
        if j==5
            [time,P]=refine(time,P,i,10);
            Dt=time(i+1)-time(i);
            qqqj=qqqi;
            qj=qi+Dt*qqi+((Dt^2)/4)*qqqi+((Dt^2)/4)*qqqj;
            qqj=qqi+(Dt/2)*qqqi+(Dt/2)*qqqj;
            j=0;
        end
        
    end
    

    vDrift_j=T*qqj;
    for jj=1:size(Fcolj,1)
        [ksl_Siv(jj)]=KS_Sivaselvan(Fcolj(jj),vDrift_j(jj),ksl(jj,jj),Fy(jj)*2,expN);
    end
    ks=T'*(ksl_Siv)*T;
    residuo(:,i+1)=P(:,i+1)-m*qqqj-cs*qqj-T'*Fdj*sind(gamma)-T'*Fcolj;
    waitbar(time(i+1)/time(end))
    
    q(:,i+1)=qj;
    qq(:,i+1)=qqj;
    qqq(:,i+1)=qqqj;
    Fd(:,i+1)=Fdj;
    Fcol(:,i+1)=Fcolj;
    
    qi=qj;
    qqi=qqj;
    qqqi=qqqj;
    Fdi=Fdj;
    Fcoli=Fcolj;
    
    if i==length(time)-1
        break
    else
        i=i+1;
    end
    
end
close(hwait)
end



function [yj]=RK_s(yi,vi,vj,ks,fy,N,dt)
vj2=(vi+vj)/2;
k0=ks*(1-(abs(yi/fy)^N)*0.5*(sign(yi*vi)+1))*vi;
k1=ks*(1-(abs((yi+dt*k0/2)/fy)^N)*0.5*(sign((yi+dt*k0/2)*vj2)+1))*vj2;
k2=ks*(1-(abs((yi+dt*k1/2)/fy)^N)*0.5*(sign((yi+dt*k1/2)*vj2)+1))*vj2;
k3=ks*(1-(abs((yi+dt*k2)/fy)^N)*0.5*(sign((yi+dt*k2)*vj)+1))*vj;
yj=yi+(dt/6)*( k0+2*k1+2*k2+k3 );
end

function [ksl_Siv]=KS_Sivaselvan(fj,vj,ks,fy,N)
ksl_Siv=ks*(1-(abs(fj/fy)^N)*0.5*(sign(fj*vj)+1));
end

function [yj]=RK_d(yi,vi,vj,alpha,kd,cd,dt)
vj2=(vi+vj)/2;
k0=(vi-sign(yi)*(abs(yi)/cd)^(1/alpha))*kd*dt;
k1=(vj2-sign(yi+k0/2)*(abs(yi+k0/2)/cd)^(1/alpha))*kd*dt;
k2=(vj2-sign(yi+k1/2)*(abs(yi+k1/2)/cd)^(1/alpha))*kd*dt;
k3=(vj-sign(yi+k2)*(abs(yi+k2)/cd)^(1/alpha))*kd*dt;
yj=yi+(1/6)*( k0+2*k1+2*k2+k3 );
end

function [time,P]=refine(time,P,i,intervalli)
timeI=time(1:i-1);
timeII=time(i+2:end);
PI=P(:,1:i-1);
PII=P(:,i+2:end);

dt=time(i+1)-time(i);
dtnew=dt/intervalli;

mP=(P(:,i+1)-P(:,i))./dt;
Ptemp=zeros(size(P,1),intervalli+1);
ttemp=zeros(1,intervalli+1);

Ptemp(1)=P(:,i);
Ptemp(end)=P(:,i+1);
ttemp(1)=time(i);
ttemp(end)=time(i+1);

for contatore=2:size(Ptemp,2)-1
    Ptemp(:,contatore)=Ptemp(:,contatore-1)+dtnew.*mP;
    ttemp(contatore)=ttemp(contatore-1)+dtnew;
end

time=[];
time=[timeI ttemp timeII];
P=[];
P=[PI Ptemp PII];
end